/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef OWL_LWIP_H
#define OWL_LWIP_H

#include <owl/core/owl_defs.h>
#include <owl/core/owl_timer.h>

/* Add common lwip includes */

#include <lwip/tcp.h>
#include <lwip/udp.h>
#include <lwip/raw.h>
#include <lwip/netif.h>
#include <lwip/dhcp.h>
#include <lwip/dns.h>
#include <lwip/ip_addr.h>
#include <lwip/udp.h>

struct owlip_ctx {
#define DHCP_STATE_IDLE      0
#define DHCP_STATE_RUN       1
#define DHCP_STATE_INFORMED  2
        uint8_t dhcp_state;

#define FLAG_DHCP      (1 << 0)
#define FLAG_DHCP_INFORM (1 << 1) /* If wakeup from hibernate */
        uint8_t flags;

        struct netif *netif;
        
        void (*addr_cb)(void *ctx, const struct ip_addr *addr);
        void *addr_ctx;

#if !OWL_CONFIG_SOCKET_BSD
        struct owltmr_tmo tcp_tmo;
        struct owltmr_tmo dhcp_tmo;
        uint32_t tick;
        uint32_t linkup_time;
#endif
    
#if OWL_CONFIG_HIBERNATE
        struct ip_addr ipaddr;
        struct ip_addr netmask;
        struct ip_addr gw;
        struct ip_addr dns;
#endif /* OWL_CONFIG_HIBERNATE */

};

extern struct owlip_ctx *owlip_ctx;


/**
 * Initialize the network stack and start the network device driver. Should be
 * called prior to any other function.
 *
 */
int owlip_init(void);

/**
 * Start TCP/IP timers explicitly. TCP/IP timers will always start upon
 * owlip_linkup().
 *
 * @note: This function is optional and is only for advanced usage.
 *
 */
void owlip_start(void);

/**
 * Stop TCP/IP timers.
 *
 * @note: This function is optional and is only for advanced usage.
 *
 */
void owlip_stop(void);

/**
 * Should be called when the network interface has link, e.g. cable inserted
 * or connected to Wi-Fi access point.
 *
 */
void owlip_linkup(void);

/** 
 * Should be called when the network interface link is down, e.g. lost
 * connection to access point.
 *
 */
void owlip_linkdown(void);

/**
 * Convert IP-stack error code to error codes defined in owl_err.h
 *
 */
int owlip_error(err_t err);

/**
 * Return the native network interface object.
 *
 */
#define owlip_netif (owlip_ctx != NULL ? owlip_ctx->netif : NULL)

#ifndef ip_ntoa /* defined by lwip-1.3.2+ but not by 1.3.0 */
# define ip_ntoa(addr) \
        ((addr != NULL) ? inet_ntoa(*((struct in_addr*)(addr))) : "NULL")
#endif
#define ip_aton ipaddr_aton

#if OWL_CONFIG_HIBERNATE
int owlip_serialize(void *data, int len);
int owlip_deserialize(const void *data, int len);
#endif /* OWL_CONFIG_HIBERNATE */

#endif /* LWIP_H */
