/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef WLP_ETHERNET_H
#define WLP_ETHERNET_H

#if defined(WLP_CONFIG_TRAMPOLINE)

void _wlp_set_rx_cb(int (*recv_cb)(void *ctx, int len), void *ctx);
int _wlp_process_tx(void *ethhdr, int len, int totlen, void *wlphdr);
int _wlp_process_rx(void *data, int len, void **stripped, int *stripped_len);
int _wlp_tx(void *data, int len);
int _wlp_rx(void *data, int len);

#else /* defined(WLP_CONFIG_TRAMPOLINE) */

#define wlp_set_rx_cb           _wlp_set_rx_cb
#define wlp_process_tx          _wlp_process_tx
#define wlp_process_rx          _wlp_process_rx
#define wlp_tx                  _wlp_tx
#define wlp_rx                  _wlp_rx

#endif /* defined(WLP_CONFIG_TRAMPOLINE) */

/**
 * Register a function that will be invoked when a new packet should be
 * delivered to the network stack. It is the responsibility of the callback
 * function to free the netbuf after processing is done - it wil no longer
 * be referenced by the driver when the recv_cb returns.
 *
 */
void wlp_set_rx_cb(int (*rx_cb)(void *ctx, int len), void *ctx);

/**
 * Buffer to wlphdr (WLP_HEADER_SIZE) should be passed to this function
 *
 * lwIP would call this function like this (if ETH_PAD_LEN is WLP_HEADER_SIZE):
 *
 * wlp_process_tx(p->payload + WLP_HEADER_SIZE,
 *                p->len - WL_HEADER_SIZE,
 *                p->tot_len - WL_HEADER_SIZE,
 *                p->payload);
 *
 * @param ethhdr is pointer to the ethernet header
 * @param len is total length of ethdr buffer
 * @param totlen is total length of packet (not including wlphdr)
 * @param wlphdr points to a buffer of WLP_HEADER_SIZE that can hold the
 *               header. Does not have to be contigous with the rest of the
 *               packet.
 */
int wlp_process_tx(void *ethhdr, int len, int totlen, void *wlphdr);

/**
 *
 * Transmit an ethernet frame.
 *
 * Can be called multiple times for each fragment that should be sent, e..g
 * each pbuf in a pbuf chain. For each packet wlp_process_tx() should be called
 * first.
 *
 * @param data should be pointing to ATH header, as obtained by wl_process_tx()
 *
 */
int wlp_tx(void *data, int len);


/**
 * Must be called in recv_cb, data should point to contig buffer holding
 * num bytes passed to recv_cb().
 */
int wlp_rx(void *data, int len);


/**
 * Get the number of bytes to strip
 *
 */
int wlp_process_rx(void *data, int len, void **stripped, int *stripped_len);

#endif /* WLP_ETHERNET_H */
